@@ -13,40 +13,44 @@ const (
 )
 
 type ChatRequest struct {
-	ContextName      string
-	SessionName      string
-	PatternName      string
-	PatternVariables map[string]string
-	Message          *chat.ChatCompletionMessage
-	Language         string
-	Meta             string
-	InputHasVars     bool
-	StrategyName     string
+	ContextName           string
+	SessionName           string
+	PatternName           string
+	PatternVariables      map[string]string
+	Message               *chat.ChatCompletionMessage
+	Language              string
+	Meta                  string
+	InputHasVars          bool
+	NoVariableReplacement bool
+	StrategyName          string
 }
 
 type ChatOptions struct {
-	Model              string
-	Temperature        float64
-	TopP               float64
-	PresencePenalty    float64
-	FrequencyPenalty   float64
-	Raw                bool
-	Seed               int
-	ModelContextLength int
-	MaxTokens          int
-	Search             bool
-	SearchLocation     string
-	ImageFile          string
-	ImageSize          string
-	ImageQuality       string
-	ImageCompression   int
-	ImageBackground    string
-	SuppressThink      bool
-	ThinkStartTag      string
-	ThinkEndTag        string
-	AudioOutput        bool
-	AudioFormat        string
-	Voice              string
+	Model               string
+	Temperature         float64
+	TopP                float64
+	PresencePenalty     float64
+	FrequencyPenalty    float64
+	Raw                 bool
+	Seed                int
+	Thinking            ThinkingLevel
+	ModelContextLength  int
+	MaxTokens           int
+	Search              bool
+	SearchLocation      string
+	ImageFile           string
+	ImageSize           string
+	ImageQuality        string
+	ImageCompression    int
+	ImageBackground     string
+	SuppressThink       bool
+	ThinkStartTag       string
+	ThinkEndTag         string
+	AudioOutput         bool
+	AudioFormat         string
+	Voice               string
+	Notification        bool
+	NotificationCommand string
 }
 
 // NormalizeMessages remove empty messages and ensure messages order user-assist-user
